<?php

declare(strict_types=1);

namespace Drupal\Tests\symfony_mailer_queue\Kernel;

use Drupal\contact\MailHandlerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Queue\DelayableQueueInterface;
use Drupal\Core\Queue\QueueGarbageCollectionInterface;
use Drupal\Core\Queue\QueueInterface;
use Drupal\KernelTests\KernelTestBase;
use Drupal\symfony_mailer\EmailFactoryInterface;
use Drupal\symfony_mailer_queue\Plugin\QueueWorker\SymfonyMailerQueueWorker;
use Drupal\symfony_mailer_test\MailerTestTrait;
use Drupal\Tests\symfony_mailer_queue\Traits\DelayableQueueRunnerTrait;
use Drupal\Tests\user\Traits\UserCreationTrait;
use Drupal\user\UserInterface;

/**
 * Queues entity emails for sending and tests send behavior.
 *
 * @group symfony_mailer_queue
 */
class SymfonyMailerQueueEntityTest extends KernelTestBase {

  use DelayableQueueRunnerTrait;
  use MailerTestTrait;
  use UserCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'contact',
    'contact_test',
    'filter',
    'symfony_mailer',
    'symfony_mailer_test',
    'symfony_mailer_queue',
    'symfony_mailer_queue_test',
    'system',
    'user',
  ];

  /**
   * The email factory.
   */
  protected EmailFactoryInterface $emailFactory;

  /**
   * The entity type manager.
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * The contact mail handler.
   */
  protected MailHandlerInterface $mailHandler;

  /**
   * The Symfony mailer queue.
   */
  protected QueueInterface $queue;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {

    parent::setUp();

    $this->installConfig([
      'contact',
      'contact_test',
      'symfony_mailer',
      'symfony_mailer_queue',
    ]);
    $this->installEntitySchema('user');

    $this->config('system.site')
      ->set('name', 'Example')
      ->set('mail', 'site@example.com')
      ->save();

    $this->emailFactory = $this->container->get('email_factory');
    $this->entityTypeManager = $this->container->get('entity_type.manager');
    $this->mailHandler = $this->container->get('contact.mail_handler');

    /** @var \Drupal\Core\Queue\QueueFactory $queue_factory */
    $queue_factory = $this->container->get('queue');
    $this->queue = $queue_factory->get(SymfonyMailerQueueWorker::QUEUE_NAME);
    $this->queue->createQueue();
    $this->assertInstanceOf(DelayableQueueInterface::class, $this->queue);
    $this->assertInstanceOf(QueueGarbageCollectionInterface::class, $this->queue);

    /** @var \Drupal\symfony_mailer\Processor\OverrideManagerInterface $override_manager */
    $override_manager = $this->container->get('symfony_mailer.override_manager');
    $override_manager->action('contact_form', 'enable');

    // The config overrider resets the configuration. Therefore, the test config
    // needs to be installed last.
    $this->installConfig(['symfony_mailer_queue_test']);
  }

  /**
   * Tests the entity email queue behavior.
   */
  public function testEntityEmailQueue(): void {

    $user = $this->createUser(['access site-wide contact form'], NULL, FALSE, ['uid' => 2]);
    $this->assertInstanceOf(UserInterface::class, $user);

    /** @var \Drupal\contact\Entity\Message $message */
    $message = $this->entityTypeManager
      ->getStorage('contact_message')
      ->create(['contact_form' => 'feedback']);
    $message->setMessage('Test Message');
    $message->setSenderMail($user->getEmail() ?? '');
    $message->save();

    $this->mailHandler->sendMailMessages($message, $user);

    $this->assertEquals(1, $this->queue->numberOfItems());
    $this->runQueue(SymfonyMailerQueueWorker::QUEUE_NAME);
    $this->assertEquals(0, $this->queue->numberOfItems());

    $this->readMail();
    $this->assertNoError();

    $this->assertSubject('Symfony Mailer Queue Entity Test');
    $this->assertAddress('reply-to', $user->getEmail());
    $this->assertAddress('to', '<site>');
    $this->assertBodyContains('Test Message');
  }

}
